import gsound_lex
import sys

PASSTHROUGH_TOKENS = [
	'OCTAVE',
	'VOLUME',
	'LOOPMARKER',
	'TOGGLEOUTPUT',
]

DURATION_TOKENS = [
	'NOTE',
	'REST',
	'NOISE',
]

class Overlay(object):
	def __init__(self):
		self.commands = []
		self.duration = 0

def totuple(tok):
	return (tok.type, tok.value)

def main():
	if len(sys.argv) < 3:
		print('Usage: {} <filename> <out>'.format(sys.argv[0]))
		sys.exit(1)

	with open(sys.argv[1], 'r') as f:
		contents = f.read()

	commands = []
	octave = None
	volume = 'a'
	use_overlay = None

	overlays = {}
	def_overlay = None

	gsound_lex.lexer.input(contents)
	for tok in gsound_lex.lexer:
		print(tok)
		if def_overlay:
			if tok.type == 'ENDDEFOVERLAY':
				def_overlay = None
			else:
				overlay = overlays[def_overlay]
				overlay.commands.append(totuple(tok))
				if tok.type in DURATION_TOKENS:
					overlay.duration += tok.value[1]
		else:
			if tok.type in PASSTHROUGH_TOKENS:
				commands.append(totuple(tok))

				if tok.type == 'OCTAVE':
					octave = tok.value
				elif tok.type == 'VOLUME':
					volume = tok.value
			elif tok.type in DURATION_TOKENS:
				if use_overlay:
					overlay = overlays[use_overlay]
					commands.extend(overlay.commands)
					commands.append(('OCTAVE', octave))
					commands.append(('VOLUME', volume))

					note, duration = tok.value
					duration -= overlay.duration
					commands.append((tok.type, (note, duration)))

					use_overlay = None
				else:
					commands.append(totuple(tok))
			elif tok.type == 'OVERLAY':
				use_overlay = tok.value
			elif tok.type == 'DEFOVERLAY':
				def_overlay = tok.value
				overlays[def_overlay] = Overlay()
			else:
				raise RuntimeError('unexpected tok {} in parse => gcmd'.format(tok))

	print(commands)

	volume = 'a'
	octave = None
	last_duration = 0
	enable_output = True

	with open(sys.argv[2], 'w') as f:
		for command, value in commands:
			if command == 'OCTAVE':
				octave = value
			elif command == 'VOLUME':
				volume = value
			elif command == 'TOGGLEOUTPUT':
				enable_output = not enable_output
			elif command in DURATION_TOKENS:
				note_name, duration = value
				fs_note_name = note_name[0].upper() if note_name[-1] == 's' else note_name
				fs_octave = octave if command == 'NOTE' else '-'
				fs_vol = volume if command != 'REST' else ''

				dur_cmd = 't{};'.format(duration) if last_duration != duration else ''
				note_cmd = '{}{}f{}'.format(fs_note_name, fs_octave, fs_vol)
				if enable_output:
					f.write(dur_cmd + note_cmd + '\n')

				last_duration = duration
			elif command == 'LOOPMARKER':
				if enable_output:
					f.write(value + '\n')
			else:
				raise RuntimeError('unexpected cmd ({}, {}) in gcmd => fcmd'.format(command, value))

if __name__ == '__main__':
	main()
