import ply.lex as lex
import re

tokens = (
	'OCTAVE',	# o3
	'VOLUME',	# v5
	'DEFOVERLAY',	# =K
	'ENDDEFOVERLAY',	# ;
	'TOGGLEOUTPUT',	# %
	'OVERLAY',	# +K
	'NOTE',	# a10, cs40, etc
	'REST',	# r20
	'NOISE',	# x20
	'LOOPMARKER',	# L4, LS
)

t_ENDDEFOVERLAY = r';'
t_TOGGLEOUTPUT = r'%'
t_LOOPMARKER = r'L[S2-9]'

def t_OCTAVE(t):
	r'o[1-7]'
	t.value = t.value[1]
	return t

def t_VOLUME(t):
	r'v[0-9a-f]'
	t.value = t.value[1]
	return t

def t_DEFOVERLAY(t):
	r'=[A-Z]'
	t.value = t.value[1]
	return t

def t_OVERLAY(t):
	r'\+[A-Z]'
	t.value = t.value[1]
	return t

def t_NOTE(t):
	r'([cdfga]s?|[eb])\d+'
	note_name = t.value[0]
	if t.value[1] == 's':
		note_name += 's'
		note_duration = int(t.value[2:])
	else:
		note_duration = int(t.value[1:])

	t.value = (note_name, note_duration)
	return t

def t_REST(t):
	r'r\d+'
	t.value = ('r', int(t.value[1:]))
	return t

def t_NOISE(t):
	r'x\d+'
	t.value = ('x', int(t.value[1:]))
	return t

def t_error(t):
	print('uhhhh bork? ({})'.format(t.value[0]))
	t.lexer.skip(1)

t_ignore_COMMENT = r'\#.*'
t_ignore = ' \t\n'

lexer = lex.lex()
